# -*- coding: utf-8 -*-
# BSD 3-Clause License
#
# Copyright (c) 2017, Nokia
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
# * Neither the name of the copyright holder nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""
nuage-vsd-sim
=============
A sample Nuage VSD API simulator
"""

import argparse
import ConfigParser
import os

from flask import Flask
from flask_restful import Api

from nuagevsdsim import simentities as sim
from nuagevsdsim.common import NUSimConfig, utils


class NuageVSDSim(object):
    def __init__(self):
        """
        Handle commands for interaction between Nuage Networks VSP and Amazon AWS
        """

        parser = argparse.ArgumentParser(
            description="A sample Nuage VSD API simulator."
        )
        parser.add_argument('-c', '--config-file', required=False,
                            help='Configuration file to use, if not specified ~/.nuage-vsd-sim/config.ini is used, it that does not exist, /etc/nuage-vsd-sim/config.ini is used.',
                            dest='config_file', type=str)
        parser.add_argument('-p', '--port', required=False, help='Port to listen on', dest='port', type=int, default=5000)
        parser.add_argument('-l', '--log-level', required=False, help='Loglevel, supported values are DEBUG, INFO, WARNING, ERROR, CRITICAL.', dest='log_level', type=str, 
                            choices=['DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL'], default='WARNING')
        args, command_args = parser.parse_known_args()

        # Handling configuration file
        if args.config_file:
            cfg = utils.parse_config(args.config_file)
        elif os.path.isfile('{0:s}/.nuage-vsd-sim/config.ini'.format(os.path.expanduser('~'))):
            cfg = utils.parse_config('{0:s}/.nuage-vsd-sim/config.ini'.format(os.path.expanduser('~')))
        elif os.path.isfile('/etc/nuage-vsd-sim/config.ini'):
            cfg = utils.parse_config('/etc/nuage-vsd-sim/config.ini')
        else:
            cfg = ConfigParser.ConfigParser()
            cfg.add_section('LOG')
            cfg.set('LOG', 'directory', '')
            cfg.set('LOG', 'file', '')
            cfg.set('LOG', 'level', args.log_level)

        # Handling logging
        log_dir = cfg.get('LOG', 'directory')
        log_file = cfg.get('LOG', 'file')
        log_level = cfg.get('LOG', 'level')

        if not log_level:
            log_level = 'WARNING'

        log_path = None
        if log_dir and log_file and os.path.isdir(log_dir) and os.access(log_dir, os.W_OK):
            log_path = os.path.join(log_dir, log_file)

        logger = utils.configure_logging(log_level, log_path)
        logger.debug('Logging initiated')

        utils.init_base_entities()

        self.app = Flask(__name__)

        self.app.config.from_object(NUSimConfig)

        self.api = Api(self.app)

        self.api.add_resource(
            sim.NUSimRoot,
            '/',
            '/nuage',
            '/nuage/api',
            '/nuage/api/v5_0'
        )

        self.api.add_resource(
            sim.NUSimAddressMap,
            '/nuage/api/v5_0/addressmaps',
            '/nuage/api/v5_0/addressmaps/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/addressmaps'
        )
        self.api.add_resource(
            sim.NUSimAddressRange,
            '/nuage/api/v5_0/addressranges',
            '/nuage/api/v5_0/addressranges/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/addressranges'
        )
        self.api.add_resource(
            sim.NUSimAggregateMetadata,
            '/nuage/api/v5_0/aggregatemetadatas',
            '/nuage/api/v5_0/aggregatemetadatas/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/aggregatemetadatas'
        )
        self.api.add_resource(
            sim.NUSimAlarm,
            '/nuage/api/v5_0/alarms',
            '/nuage/api/v5_0/alarms/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/alarms'
        )
        self.api.add_resource(
            sim.NUSimAllAlarm,
            '/nuage/api/v5_0/allalarms',
            '/nuage/api/v5_0/allalarms/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/allalarms'
        )
        self.api.add_resource(
            sim.NUSimApplicationBinding,
            '/nuage/api/v5_0/applicationbindings',
            '/nuage/api/v5_0/applicationbindings/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/applicationbindings'
        )
        self.api.add_resource(
            sim.NUSimApplicationperformancemanagementbinding,
            '/nuage/api/v5_0/applicationperformancemanagementbindings',
            '/nuage/api/v5_0/applicationperformancemanagementbindings/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/applicationperformancemanagementbindings'
        )
        self.api.add_resource(
            sim.NUSimApplicationperformancemanagement,
            '/nuage/api/v5_0/applicationperformancemanagements',
            '/nuage/api/v5_0/applicationperformancemanagements/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/applicationperformancemanagements'
        )
        self.api.add_resource(
            sim.NUSimApplication,
            '/nuage/api/v5_0/applications',
            '/nuage/api/v5_0/applications/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/applications'
        )
        self.api.add_resource(
            sim.NUSimAutoDiscoverCluster,
            '/nuage/api/v5_0/autodiscoveredclusters',
            '/nuage/api/v5_0/autodiscoveredclusters/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/autodiscoveredclusters'
        )
        self.api.add_resource(
            sim.NUSimAutodiscovereddatacenter,
            '/nuage/api/v5_0/autodiscovereddatacenters',
            '/nuage/api/v5_0/autodiscovereddatacenters/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/autodiscovereddatacenters'
        )
        self.api.add_resource(
            sim.NUSimAutoDiscoveredGateway,
            '/nuage/api/v5_0/autodiscoveredgateways',
            '/nuage/api/v5_0/autodiscoveredgateways/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/autodiscoveredgateways'
        )
        self.api.add_resource(
            sim.NUSimAutoDiscoverHypervisorFromCluster,
            '/nuage/api/v5_0/autodiscoveredhypervisors',
            '/nuage/api/v5_0/autodiscoveredhypervisors/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/autodiscoveredhypervisors'
        )
        self.api.add_resource(
            sim.NUSimAvatar,
            '/nuage/api/v5_0/avatars',
            '/nuage/api/v5_0/avatars/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/avatars'
        )
        self.api.add_resource(
            sim.NUSimBFDSession,
            '/nuage/api/v5_0/bfdsessions',
            '/nuage/api/v5_0/bfdsessions/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/bfdsessions'
        )
        self.api.add_resource(
            sim.NUSimBGPNeighbor,
            '/nuage/api/v5_0/bgpneighbors',
            '/nuage/api/v5_0/bgpneighbors/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/bgpneighbors'
        )
        self.api.add_resource(
            sim.NUSimBGPPeer,
            '/nuage/api/v5_0/bgppeers',
            '/nuage/api/v5_0/bgppeers/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/bgppeers'
        )
        self.api.add_resource(
            sim.NUSimBGPProfile,
            '/nuage/api/v5_0/bgpprofiles',
            '/nuage/api/v5_0/bgpprofiles/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/bgpprofiles'
        )
        self.api.add_resource(
            sim.NUSimBootstrapActivation,
            '/nuage/api/v5_0/bootstrapactivations',
            '/nuage/api/v5_0/bootstrapactivations/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/bootstrapactivations'
        )
        self.api.add_resource(
            sim.NUSimBootstrap,
            '/nuage/api/v5_0/bootstraps',
            '/nuage/api/v5_0/bootstraps/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/bootstraps'
        )
        self.api.add_resource(
            sim.NUSimBRConnection,
            '/nuage/api/v5_0/brconnections',
            '/nuage/api/v5_0/brconnections/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/brconnections'
        )
        self.api.add_resource(
            sim.NUSimBridgeInterface,
            '/nuage/api/v5_0/bridgeinterfaces',
            '/nuage/api/v5_0/bridgeinterfaces/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/bridgeinterfaces'
        )
        self.api.add_resource(
            sim.NUSimBulkStatistics,
            '/nuage/api/v5_0/bulkstatistics',
            '/nuage/api/v5_0/bulkstatistics/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/bulkstatistics'
        )
        self.api.add_resource(
            sim.NUSimCaptivePortalProfile,
            '/nuage/api/v5_0/captiveportalprofiles',
            '/nuage/api/v5_0/captiveportalprofiles/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/captiveportalprofiles'
        )
        self.api.add_resource(
            sim.NUSimCertificate,
            '/nuage/api/v5_0/certificates',
            '/nuage/api/v5_0/certificates/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/certificates'
        )
        self.api.add_resource(
            sim.NUSimCloudMgmtSystem,
            '/nuage/api/v5_0/cms',
            '/nuage/api/v5_0/cms/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/cms'
        )
        self.api.add_resource(
            sim.NUSimCommand,
            '/nuage/api/v5_0/commands',
            '/nuage/api/v5_0/commands/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/commands'
        )
        self.api.add_resource(
            sim.NUSimVSDComponent,
            '/nuage/api/v5_0/components',
            '/nuage/api/v5_0/components/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/components'
        )
        self.api.add_resource(
            sim.NUSimConnectionendpoint,
            '/nuage/api/v5_0/connectionendpoints',
            '/nuage/api/v5_0/connectionendpoints/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/connectionendpoints'
        )
        self.api.add_resource(
            sim.NUSimContainerInterface,
            '/nuage/api/v5_0/containerinterfaces',
            '/nuage/api/v5_0/containerinterfaces/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/containerinterfaces'
        )
        self.api.add_resource(
            sim.NUSimContainerResync,
            '/nuage/api/v5_0/containerresync',
            '/nuage/api/v5_0/containerresync/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/containerresync'
        )
        self.api.add_resource(
            sim.NUSimContainer,
            '/nuage/api/v5_0/containers',
            '/nuage/api/v5_0/containers/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/containers'
        )
        self.api.add_resource(
            sim.NUSimCOSRemarkingPolicy,
            '/nuage/api/v5_0/cosremarkingpolicies',
            '/nuage/api/v5_0/cosremarkingpolicies/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/cosremarkingpolicies'
        )
        self.api.add_resource(
            sim.NUSimCOSRemarkingPolicyTable,
            '/nuage/api/v5_0/cosremarkingpolicytables',
            '/nuage/api/v5_0/cosremarkingpolicytables/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/cosremarkingpolicytables'
        )
        self.api.add_resource(
            sim.NUSimCSNATPool,
            '/nuage/api/v5_0/csnatpools',
            '/nuage/api/v5_0/csnatpools/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/csnatpools'
        )
        self.api.add_resource(
            sim.NUSimCTranslationMap,
            '/nuage/api/v5_0/ctranslationmaps',
            '/nuage/api/v5_0/ctranslationmaps/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/ctranslationmaps'
        )
        self.api.add_resource(
            sim.NUSimCustomProperty,
            '/nuage/api/v5_0/customproperties',
            '/nuage/api/v5_0/customproperties/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/customproperties'
        )
        self.api.add_resource(
            sim.NUSimDemarcationService,
            '/nuage/api/v5_0/demarcationservices',
            '/nuage/api/v5_0/demarcationservices/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/demarcationservices'
        )
        self.api.add_resource(
            sim.NUSimDestinationurl,
            '/nuage/api/v5_0/destinationurls',
            '/nuage/api/v5_0/destinationurls/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/destinationurls'
        )
        self.api.add_resource(
            sim.NUSimDHCPOption,
            '/nuage/api/v5_0/dhcpoptions',
            '/nuage/api/v5_0/dhcpoptions/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/dhcpoptions'
        )
        self.api.add_resource(
            sim.NUSimDiskStat,
            '/nuage/api/v5_0/diskstats',
            '/nuage/api/v5_0/diskstats/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/diskstats'
        )
        self.api.add_resource(
            sim.NUSimDomain,
            '/nuage/api/v5_0/domains',
            '/nuage/api/v5_0/domains/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/domains'
        )
        self.api.add_resource(
            sim.NUSimDomainTemplate,
            '/nuage/api/v5_0/domaintemplates',
            '/nuage/api/v5_0/domaintemplates/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/domaintemplates'
        )
        self.api.add_resource(
            sim.NUSimDSCPForwardingClassMapping,
            '/nuage/api/v5_0/dscpforwardingclassmappings',
            '/nuage/api/v5_0/dscpforwardingclassmappings/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/dscpforwardingclassmappings'
        )
        self.api.add_resource(
            sim.NUSimDSCPForwardingClassTable,
            '/nuage/api/v5_0/dscpforwardingclasstables',
            '/nuage/api/v5_0/dscpforwardingclasstables/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/dscpforwardingclasstables'
        )
        self.api.add_resource(
            sim.NUSimDSCPRemarkingPolicy,
            '/nuage/api/v5_0/dscpremarkingpolicies',
            '/nuage/api/v5_0/dscpremarkingpolicies/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/dscpremarkingpolicies'
        )
        self.api.add_resource(
            sim.NUSimDSCPRemarkingPolicyTable,
            '/nuage/api/v5_0/dscpremarkingpolicytables',
            '/nuage/api/v5_0/dscpremarkingpolicytables/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/dscpremarkingpolicytables'
        )
        self.api.add_resource(
            sim.NUSimDUCGroupBinding,
            '/nuage/api/v5_0/ducgroupbindings',
            '/nuage/api/v5_0/ducgroupbindings/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/ducgroupbindings'
        )
        self.api.add_resource(
            sim.NUSimDUCGroup,
            '/nuage/api/v5_0/ducgroups',
            '/nuage/api/v5_0/ducgroups/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/ducgroups'
        )
        self.api.add_resource(
            sim.NUSimVCenterEAMConfig,
            '/nuage/api/v5_0/eamconfigs',
            '/nuage/api/v5_0/eamconfigs/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/eamconfigs'
        )
        self.api.add_resource(
            sim.NUSimEgressACLEntryTemplate,
            '/nuage/api/v5_0/egressaclentrytemplates',
            '/nuage/api/v5_0/egressaclentrytemplates/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/egressaclentrytemplates'
        )
        self.api.add_resource(
            sim.NUSimEgressACLTemplate,
            '/nuage/api/v5_0/egressacltemplates',
            '/nuage/api/v5_0/egressacltemplates/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/egressacltemplates'
        )
        self.api.add_resource(
            sim.NUSimEgressAdvFwdEntryTemplate,
            '/nuage/api/v5_0/egressadvfwdentrytemplates',
            '/nuage/api/v5_0/egressadvfwdentrytemplates/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/egressadvfwdentrytemplates'
        )
        self.api.add_resource(
            sim.NUSimEgressAdvFwdTemplate,
            '/nuage/api/v5_0/egressadvfwdtemplates',
            '/nuage/api/v5_0/egressadvfwdtemplates/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/egressadvfwdtemplates'
        )
        self.api.add_resource(
            sim.NUSimDomainFIPAclTemplateEntry,
            '/nuage/api/v5_0/egressdomainfloatingipaclentrytemplates',
            '/nuage/api/v5_0/egressdomainfloatingipaclentrytemplates/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/egressdomainfloatingipaclentrytemplates'
        )
        self.api.add_resource(
            sim.NUSimDomainFIPAclTemplate,
            '/nuage/api/v5_0/egressdomainfloatingipacltemplates',
            '/nuage/api/v5_0/egressdomainfloatingipacltemplates/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/egressdomainfloatingipacltemplates'
        )
        self.api.add_resource(
            sim.NUSimFloatingIPACLTemplateEntry,
            '/nuage/api/v5_0/egressfloatingipaclentrytemplates',
            '/nuage/api/v5_0/egressfloatingipaclentrytemplates/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/egressfloatingipaclentrytemplates'
        )
        self.api.add_resource(
            sim.NUSimFloatingIPACLTemplate,
            '/nuage/api/v5_0/egressfloatingipacltemplates',
            '/nuage/api/v5_0/egressfloatingipacltemplates/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/egressfloatingipacltemplates'
        )
        self.api.add_resource(
            sim.NUSimEgressQOSPolicy,
            '/nuage/api/v5_0/egressqospolicies',
            '/nuage/api/v5_0/egressqospolicies/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/egressqospolicies'
        )
        self.api.add_resource(
            sim.NUSimEnterpriseNetwork,
            '/nuage/api/v5_0/enterprisenetworks',
            '/nuage/api/v5_0/enterprisenetworks/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/enterprisenetworks'
        )
        self.api.add_resource(
            sim.NUSimEnterprisePermission,
            '/nuage/api/v5_0/enterprisepermissions',
            '/nuage/api/v5_0/enterprisepermissions/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/enterprisepermissions'
        )
        self.api.add_resource(
            sim.NUSimEnterpriseProfile,
            '/nuage/api/v5_0/enterpriseprofiles',
            '/nuage/api/v5_0/enterpriseprofiles/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/enterpriseprofiles'
        )
        self.api.add_resource(
            sim.NUSimEnterprise,
            '/nuage/api/v5_0/enterprises',
            '/nuage/api/v5_0/enterprises/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/enterprises'
        )
        self.api.add_resource(
            sim.NUSimEnterpriseSecuredData,
            '/nuage/api/v5_0/enterprisesecureddatas',
            '/nuage/api/v5_0/enterprisesecureddatas/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/enterprisesecureddatas'
        )
        self.api.add_resource(
            sim.NUSimEnterpriseSecurity,
            '/nuage/api/v5_0/enterprisesecurities',
            '/nuage/api/v5_0/enterprisesecurities/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/enterprisesecurities'
        )
        self.api.add_resource(
            sim.NUSimEventLog,
            '/nuage/api/v5_0/eventlogs',
            '/nuage/api/v5_0/eventlogs/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/eventlogs'
        )
        self.api.add_resource(
            sim.NUSimFirewallAcl,
            '/nuage/api/v5_0/firewallacls',
            '/nuage/api/v5_0/firewallacls/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/firewallacls'
        )
        self.api.add_resource(
            sim.NUSimFirewallRule,
            '/nuage/api/v5_0/firewallrules',
            '/nuage/api/v5_0/firewallrules/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/firewallrules'
        )
        self.api.add_resource(
            sim.NUSimFloatingIp,
            '/nuage/api/v5_0/floatingips',
            '/nuage/api/v5_0/floatingips/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/floatingips'
        )
        self.api.add_resource(
            sim.NUSimGateway,
            '/nuage/api/v5_0/gateways',
            '/nuage/api/v5_0/gateways/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/gateways'
        )
        self.api.add_resource(
            sim.NUSimGatewaySecuredData,
            '/nuage/api/v5_0/gatewaysecureddatas',
            '/nuage/api/v5_0/gatewaysecureddatas/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/gatewaysecureddatas'
        )
        self.api.add_resource(
            sim.NUSimGatewaySecurity,
            '/nuage/api/v5_0/gatewaysecurities',
            '/nuage/api/v5_0/gatewaysecurities/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/gatewaysecurities'
        )
        self.api.add_resource(
            sim.NUSimGatewayTemplate,
            '/nuage/api/v5_0/gatewaytemplates',
            '/nuage/api/v5_0/gatewaytemplates/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/gatewaytemplates'
        )
        self.api.add_resource(
            sim.NUSimGlobalMetadata,
            '/nuage/api/v5_0/globalmetadatas',
            '/nuage/api/v5_0/globalmetadatas/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/globalmetadatas'
        )
        self.api.add_resource(
            sim.NUSimGroupKeyEncryptionProfile,
            '/nuage/api/v5_0/groupkeyencryptionprofiles',
            '/nuage/api/v5_0/groupkeyencryptionprofiles/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/groupkeyencryptionprofiles'
        )
        self.api.add_resource(
            sim.NUSimGroup,
            '/nuage/api/v5_0/groups',
            '/nuage/api/v5_0/groups/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/groups'
        )
        self.api.add_resource(
            sim.NUSimHostInterface,
            '/nuage/api/v5_0/hostinterfaces',
            '/nuage/api/v5_0/hostinterfaces/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/hostinterfaces'
        )
        self.api.add_resource(
            sim.NUSimHSC,
            '/nuage/api/v5_0/hscs',
            '/nuage/api/v5_0/hscs/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/hscs'
        )
        self.api.add_resource(
            sim.NUSimIKECertificate,
            '/nuage/api/v5_0/ikecertificates',
            '/nuage/api/v5_0/ikecertificates/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/ikecertificates'
        )
        self.api.add_resource(
            sim.NUSimIKEEncryptionprofile,
            '/nuage/api/v5_0/ikeencryptionprofiles',
            '/nuage/api/v5_0/ikeencryptionprofiles/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/ikeencryptionprofiles'
        )
        self.api.add_resource(
            sim.NUSimIKEGatewayConfig,
            '/nuage/api/v5_0/ikegatewayconfig',
            '/nuage/api/v5_0/ikegatewayconfig/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/ikegatewayconfig'
        )
        self.api.add_resource(
            sim.NUSimIKEGatewayConnection,
            '/nuage/api/v5_0/ikegatewayconnections',
            '/nuage/api/v5_0/ikegatewayconnections/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/ikegatewayconnections'
        )
        self.api.add_resource(
            sim.NUSimIKEGatewayProfile,
            '/nuage/api/v5_0/ikegatewayprofiles',
            '/nuage/api/v5_0/ikegatewayprofiles/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/ikegatewayprofiles'
        )
        self.api.add_resource(
            sim.NUSimIKEGateway,
            '/nuage/api/v5_0/ikegateways',
            '/nuage/api/v5_0/ikegateways/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/ikegateways'
        )
        self.api.add_resource(
            sim.NUSimIKEPSK,
            '/nuage/api/v5_0/ikepsks',
            '/nuage/api/v5_0/ikepsks/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/ikepsks'
        )
        self.api.add_resource(
            sim.NUSimIKESubnet,
            '/nuage/api/v5_0/ikesubnets',
            '/nuage/api/v5_0/ikesubnets/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/ikesubnets'
        )
        self.api.add_resource(
            sim.NUSimInfrastructureConfig,
            '/nuage/api/v5_0/infraconfig',
            '/nuage/api/v5_0/infraconfig/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/infraconfig'
        )
        self.api.add_resource(
            sim.NUSimInfrastructureAccessProfile,
            '/nuage/api/v5_0/infrastructureaccessprofiles',
            '/nuage/api/v5_0/infrastructureaccessprofiles/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/infrastructureaccessprofiles'
        )
        self.api.add_resource(
            sim.NUSimInfrastructureGatewayProfile,
            '/nuage/api/v5_0/infrastructuregatewayprofiles',
            '/nuage/api/v5_0/infrastructuregatewayprofiles/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/infrastructuregatewayprofiles'
        )
        self.api.add_resource(
            sim.NUSimInfrastructureVscProfile,
            '/nuage/api/v5_0/infrastructurevscprofiles',
            '/nuage/api/v5_0/infrastructurevscprofiles/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/infrastructurevscprofiles'
        )
        self.api.add_resource(
            sim.NUSimIngressACLEntryTemplate,
            '/nuage/api/v5_0/ingressaclentrytemplates',
            '/nuage/api/v5_0/ingressaclentrytemplates/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/ingressaclentrytemplates'
        )
        self.api.add_resource(
            sim.NUSimIngressACLTemplate,
            '/nuage/api/v5_0/ingressacltemplates',
            '/nuage/api/v5_0/ingressacltemplates/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/ingressacltemplates'
        )
        self.api.add_resource(
            sim.NUSimIngressAdvFwdEntryTemplate,
            '/nuage/api/v5_0/ingressadvfwdentrytemplates',
            '/nuage/api/v5_0/ingressadvfwdentrytemplates/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/ingressadvfwdentrytemplates'
        )
        self.api.add_resource(
            sim.NUSimIngressAdvFwdTemplate,
            '/nuage/api/v5_0/ingressadvfwdtemplates',
            '/nuage/api/v5_0/ingressadvfwdtemplates/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/ingressadvfwdtemplates'
        )
        self.api.add_resource(
            sim.NUSimIngressExternalServiceTemplateEntry,
            '/nuage/api/v5_0/ingressexternalserviceentrytemplates',
            '/nuage/api/v5_0/ingressexternalserviceentrytemplates/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/ingressexternalserviceentrytemplates'
        )
        self.api.add_resource(
            sim.NUSimIngressExternalServiceTemplate,
            '/nuage/api/v5_0/ingressexternalservicetemplates',
            '/nuage/api/v5_0/ingressexternalservicetemplates/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/ingressexternalservicetemplates'
        )
        self.api.add_resource(
            sim.NUSimIngressQOSPolicy,
            '/nuage/api/v5_0/ingressqospolicies',
            '/nuage/api/v5_0/ingressqospolicies/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/ingressqospolicies'
        )
        self.api.add_resource(
            sim.NUSimIPReservation,
            '/nuage/api/v5_0/ipreservations',
            '/nuage/api/v5_0/ipreservations/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/ipreservations'
        )
        self.api.add_resource(
            sim.NUSimJob,
            '/nuage/api/v5_0/jobs',
            '/nuage/api/v5_0/jobs/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/jobs'
        )
        self.api.add_resource(
            sim.NUSimKeyServerMember,
            '/nuage/api/v5_0/keyservermembers',
            '/nuage/api/v5_0/keyservermembers/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/keyservermembers'
        )
        self.api.add_resource(
            sim.NUSimKeyServerMonitorEncryptedSeed,
            '/nuage/api/v5_0/keyservermonitorencryptedseeds',
            '/nuage/api/v5_0/keyservermonitorencryptedseeds/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/keyservermonitorencryptedseeds'
        )
        self.api.add_resource(
            sim.NUSimKeyServerMonitor,
            '/nuage/api/v5_0/keyservermonitors',
            '/nuage/api/v5_0/keyservermonitors/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/keyservermonitors'
        )
        self.api.add_resource(
            sim.NUSimKeyServerMonitorSeed,
            '/nuage/api/v5_0/keyservermonitorseeds',
            '/nuage/api/v5_0/keyservermonitorseeds/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/keyservermonitorseeds'
        )
        self.api.add_resource(
            sim.NUSimKeyServerMonitorSEK,
            '/nuage/api/v5_0/keyservermonitorseks',
            '/nuage/api/v5_0/keyservermonitorseks/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/keyservermonitorseks'
        )
        self.api.add_resource(
            sim.NUSimL2Domain,
            '/nuage/api/v5_0/l2domains',
            '/nuage/api/v5_0/l2domains/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/l2domains'
        )
        self.api.add_resource(
            sim.NUSimL2DomainTemplate,
            '/nuage/api/v5_0/l2domaintemplates',
            '/nuage/api/v5_0/l2domaintemplates/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/l2domaintemplates'
        )
        self.api.add_resource(
            sim.NUSimL4ServiceGroup,
            '/nuage/api/v5_0/l4servicegroups',
            '/nuage/api/v5_0/l4servicegroups/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/l4servicegroups'
        )
        self.api.add_resource(
            sim.NUSimL4Service,
            '/nuage/api/v5_0/l4services',
            '/nuage/api/v5_0/l4services/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/l4services'
        )
        self.api.add_resource(
            sim.NUSimL7applicationsignature,
            '/nuage/api/v5_0/l7applicationsignatures',
            '/nuage/api/v5_0/l7applicationsignatures/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/l7applicationsignatures'
        )
        self.api.add_resource(
            sim.NUSimLDAPConfiguration,
            '/nuage/api/v5_0/ldapconfigurations',
            '/nuage/api/v5_0/ldapconfigurations/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/ldapconfigurations'
        )
        self.api.add_resource(
            sim.NUSimLicense,
            '/nuage/api/v5_0/licenses',
            '/nuage/api/v5_0/licenses/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/licenses'
        )
        self.api.add_resource(
            sim.NUSimLicenseStatus,
            '/nuage/api/v5_0/licensestatus',
            '/nuage/api/v5_0/licensestatus/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/licensestatus'
        )
        self.api.add_resource(
            sim.NUSimLink,
            '/nuage/api/v5_0/links',
            '/nuage/api/v5_0/links/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/links'
        )
        self.api.add_resource(
            sim.NUSimLocation,
            '/nuage/api/v5_0/locations',
            '/nuage/api/v5_0/locations/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/locations'
        )
        self.api.add_resource(
            sim.NUSimLTEInformation,
            '/nuage/api/v5_0/lteinformations',
            '/nuage/api/v5_0/lteinformations/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/lteinformations'
        )
        self.api.add_resource(
            sim.NUSimLtestatistics,
            '/nuage/api/v5_0/ltestatistics',
            '/nuage/api/v5_0/ltestatistics/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/ltestatistics'
        )
        self.api.add_resource(
            sim.NUSimMe,
            '/nuage/api/v5_0/me'
        )
        self.api.add_resource(
            sim.NUSimMetadata,
            '/nuage/api/v5_0/metadatas',
            '/nuage/api/v5_0/metadatas/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/metadatas'
        )
        self.api.add_resource(
            sim.NUSimMirrorDestination,
            '/nuage/api/v5_0/mirrordestinations',
            '/nuage/api/v5_0/mirrordestinations/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/mirrordestinations'
        )
        self.api.add_resource(
            sim.NUSimMonitoringPort,
            '/nuage/api/v5_0/monitoringports',
            '/nuage/api/v5_0/monitoringports/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/monitoringports'
        )
        self.api.add_resource(
            sim.NUSimMonitorscope,
            '/nuage/api/v5_0/monitorscopes',
            '/nuage/api/v5_0/monitorscopes/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/monitorscopes'
        )
        self.api.add_resource(
            sim.NUSimMultiCastChannelMap,
            '/nuage/api/v5_0/multicastchannelmaps',
            '/nuage/api/v5_0/multicastchannelmaps/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/multicastchannelmaps'
        )
        self.api.add_resource(
            sim.NUSimMultiCastList,
            '/nuage/api/v5_0/multicastlists',
            '/nuage/api/v5_0/multicastlists/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/multicastlists'
        )
        self.api.add_resource(
            sim.NUSimMultiCastRange,
            '/nuage/api/v5_0/multicastranges',
            '/nuage/api/v5_0/multicastranges/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/multicastranges'
        )
        self.api.add_resource(
            sim.NUSimMultiNICVPort,
            '/nuage/api/v5_0/multinicvports',
            '/nuage/api/v5_0/multinicvports/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/multinicvports'
        )
        self.api.add_resource(
            sim.NUSimNATMapEntry,
            '/nuage/api/v5_0/natmapentries',
            '/nuage/api/v5_0/natmapentries/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/natmapentries'
        )
        self.api.add_resource(
            sim.NUSimNetworkLayout,
            '/nuage/api/v5_0/networklayout',
            '/nuage/api/v5_0/networklayout/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/networklayout'
        )
        self.api.add_resource(
            sim.NUSimNetworkMacroGroup,
            '/nuage/api/v5_0/networkmacrogroups',
            '/nuage/api/v5_0/networkmacrogroups/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/networkmacrogroups'
        )
        self.api.add_resource(
            sim.NUSimNetworkPerformanceBinding,
            '/nuage/api/v5_0/networkperformancebindings',
            '/nuage/api/v5_0/networkperformancebindings/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/networkperformancebindings'
        )
        self.api.add_resource(
            sim.NUSimNetworkPerformanceMeasurement,
            '/nuage/api/v5_0/networkperformancemeasurements',
            '/nuage/api/v5_0/networkperformancemeasurements/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/networkperformancemeasurements'
        )
        self.api.add_resource(
            sim.NUSimNextHop,
            '/nuage/api/v5_0/nexthops',
            '/nuage/api/v5_0/nexthops/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/nexthops'
        )
        self.api.add_resource(
            sim.NUSimNSGateway,
            '/nuage/api/v5_0/nsgateways',
            '/nuage/api/v5_0/nsgateways/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/nsgateways'
        )
        self.api.add_resource(
            sim.NUSimNSGatewayTemplate,
            '/nuage/api/v5_0/nsgatewaytemplates',
            '/nuage/api/v5_0/nsgatewaytemplates/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/nsgatewaytemplates'
        )
        self.api.add_resource(
            sim.NUSimNSGGroup,
            '/nuage/api/v5_0/nsggroups',
            '/nuage/api/v5_0/nsggroups/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/nsggroups'
        )
        self.api.add_resource(
            sim.NUSimNSGInfo,
            '/nuage/api/v5_0/nsginfos',
            '/nuage/api/v5_0/nsginfos/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/nsginfos'
        )
        self.api.add_resource(
            sim.NUSimNSRedundantGatewayGroup,
            '/nuage/api/v5_0/nsgredundancygroups',
            '/nuage/api/v5_0/nsgredundancygroups/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/nsgredundancygroups'
        )
        self.api.add_resource(
            sim.NUSimNSGRoutingPolicyBinding,
            '/nuage/api/v5_0/nsgroutingpolicybindings',
            '/nuage/api/v5_0/nsgroutingpolicybindings/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/nsgroutingpolicybindings'
        )
        self.api.add_resource(
            sim.NUSimNSGUpgradeProfile,
            '/nuage/api/v5_0/nsgupgradeprofiles',
            '/nuage/api/v5_0/nsgupgradeprofiles/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/nsgupgradeprofiles'
        )
        self.api.add_resource(
            sim.NUSimNSPort,
            '/nuage/api/v5_0/nsports',
            '/nuage/api/v5_0/nsports/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/nsports'
        )
        self.api.add_resource(
            sim.NUSimNSPortTemplate,
            '/nuage/api/v5_0/nsporttemplates',
            '/nuage/api/v5_0/nsporttemplates/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/nsporttemplates'
        )
        self.api.add_resource(
            sim.NUSimRedundantPort,
            '/nuage/api/v5_0/nsredundantports',
            '/nuage/api/v5_0/nsredundantports/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/nsredundantports'
        )
        self.api.add_resource(
            sim.NUSimOSPFArea,
            '/nuage/api/v5_0/ospfareas',
            '/nuage/api/v5_0/ospfareas/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/ospfareas'
        )
        self.api.add_resource(
            sim.NUSimOSPFInstance,
            '/nuage/api/v5_0/ospfinstances',
            '/nuage/api/v5_0/ospfinstances/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/ospfinstances'
        )
        self.api.add_resource(
            sim.NUSimOSPFInterface,
            '/nuage/api/v5_0/ospfinterfaces',
            '/nuage/api/v5_0/ospfinterfaces/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/ospfinterfaces'
        )
        self.api.add_resource(
            sim.NUSimOverlayAddressPool,
            '/nuage/api/v5_0/overlayaddresspools',
            '/nuage/api/v5_0/overlayaddresspools/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/overlayaddresspools'
        )
        self.api.add_resource(
            sim.NUSimOverlayMirrorDestination,
            '/nuage/api/v5_0/overlaymirrordestinations',
            '/nuage/api/v5_0/overlaymirrordestinations/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/overlaymirrordestinations'
        )
        self.api.add_resource(
            sim.NUSimOverlayMirrorDestinationTemplate,
            '/nuage/api/v5_0/overlaymirrordestinationtemplates',
            '/nuage/api/v5_0/overlaymirrordestinationtemplates/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/overlaymirrordestinationtemplates'
        )
        self.api.add_resource(
            sim.NUSimOverlayPATNATEntry,
            '/nuage/api/v5_0/overlaypatnatentries',
            '/nuage/api/v5_0/overlaypatnatentries/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/overlaypatnatentries'
        )
        self.api.add_resource(
            sim.NUSimPATIPEntry,
            '/nuage/api/v5_0/patipentries',
            '/nuage/api/v5_0/patipentries/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/patipentries'
        )
        self.api.add_resource(
            sim.NUSimPATMapper,
            '/nuage/api/v5_0/patmappers',
            '/nuage/api/v5_0/patmappers/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/patmappers'
        )
        self.api.add_resource(
            sim.NUSimPATNATPool,
            '/nuage/api/v5_0/patnatpools',
            '/nuage/api/v5_0/patnatpools/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/patnatpools'
        )
        self.api.add_resource(
            sim.NUSimPerformanceMonitor,
            '/nuage/api/v5_0/performancemonitors',
            '/nuage/api/v5_0/performancemonitors/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/performancemonitors'
        )
        self.api.add_resource(
            sim.NUSimPermission,
            '/nuage/api/v5_0/permissions',
            '/nuage/api/v5_0/permissions/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/permissions'
        )
        self.api.add_resource(
            sim.NUSimPGExpression,
            '/nuage/api/v5_0/pgexpressions',
            '/nuage/api/v5_0/pgexpressions/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/pgexpressions'
        )
        self.api.add_resource(
            sim.NUSimPGExpressionTemplate,
            '/nuage/api/v5_0/pgexpressiontemplates',
            '/nuage/api/v5_0/pgexpressiontemplates/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/pgexpressiontemplates'
        )
        self.api.add_resource(
            sim.NUSimPolicyDecision,
            '/nuage/api/v5_0/policydecisions',
            '/nuage/api/v5_0/policydecisions/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/policydecisions'
        )
        self.api.add_resource(
            sim.NUSimPolicyEntry,
            '/nuage/api/v5_0/policyentries',
            '/nuage/api/v5_0/policyentries/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/policyentries'
        )
        self.api.add_resource(
            sim.NUSimPolicyGroup,
            '/nuage/api/v5_0/policygroups',
            '/nuage/api/v5_0/policygroups/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/policygroups'
        )
        self.api.add_resource(
            sim.NUSimPolicyGroupTemplate,
            '/nuage/api/v5_0/policygrouptemplates',
            '/nuage/api/v5_0/policygrouptemplates/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/policygrouptemplates'
        )
        self.api.add_resource(
            sim.NUSimPolicyObjectGroup,
            '/nuage/api/v5_0/policyobjectgroups',
            '/nuage/api/v5_0/policyobjectgroups/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/policyobjectgroups'
        )
        self.api.add_resource(
            sim.NUSimPolicyStatement,
            '/nuage/api/v5_0/policystatements',
            '/nuage/api/v5_0/policystatements/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/policystatements'
        )
        self.api.add_resource(
            sim.NUSimPortMapping,
            '/nuage/api/v5_0/portmappings',
            '/nuage/api/v5_0/portmappings/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/portmappings'
        )
        self.api.add_resource(
            sim.NUSimPort,
            '/nuage/api/v5_0/ports',
            '/nuage/api/v5_0/ports/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/ports'
        )
        self.api.add_resource(
            sim.NUSimPortTemplate,
            '/nuage/api/v5_0/porttemplates',
            '/nuage/api/v5_0/porttemplates/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/porttemplates'
        )
        self.api.add_resource(
            sim.NUSimProxyARPFilter,
            '/nuage/api/v5_0/proxyarpfilters',
            '/nuage/api/v5_0/proxyarpfilters/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/proxyarpfilters'
        )
        self.api.add_resource(
            sim.NUSimPSNATPool,
            '/nuage/api/v5_0/psnatpools',
            '/nuage/api/v5_0/psnatpools/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/psnatpools'
        )
        self.api.add_resource(
            sim.NUSimPSPATMap,
            '/nuage/api/v5_0/pspatmaps',
            '/nuage/api/v5_0/pspatmaps/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/pspatmaps'
        )
        self.api.add_resource(
            sim.NUSimPTranslationMap,
            '/nuage/api/v5_0/ptranslationmaps',
            '/nuage/api/v5_0/ptranslationmaps/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/ptranslationmaps'
        )
        self.api.add_resource(
            sim.NUSimPublicNetworkMacro,
            '/nuage/api/v5_0/publicnetworks',
            '/nuage/api/v5_0/publicnetworks/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/publicnetworks'
        )
        self.api.add_resource(
            sim.NUSimQOS,
            '/nuage/api/v5_0/qos',
            '/nuage/api/v5_0/qos/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/qos'
        )
        self.api.add_resource(
            sim.NUSimQosPolicer,
            '/nuage/api/v5_0/qospolicers',
            '/nuage/api/v5_0/qospolicers/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/qospolicers'
        )
        self.api.add_resource(
            sim.NUSimRateLimiter,
            '/nuage/api/v5_0/ratelimiters',
            '/nuage/api/v5_0/ratelimiters/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/ratelimiters'
        )
        self.api.add_resource(
            sim.NUSimRedirectionTarget,
            '/nuage/api/v5_0/redirectiontargets',
            '/nuage/api/v5_0/redirectiontargets/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/redirectiontargets'
        )
        self.api.add_resource(
            sim.NUSimRedirectionTargetTemplate,
            '/nuage/api/v5_0/redirectiontargettemplates',
            '/nuage/api/v5_0/redirectiontargettemplates/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/redirectiontargettemplates'
        )
        self.api.add_resource(
            sim.NUSimRedundancyGroup,
            '/nuage/api/v5_0/redundancygroups',
            '/nuage/api/v5_0/redundancygroups/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/redundancygroups'
        )
        self.api.add_resource(
            sim.NUSimVMResync,
            '/nuage/api/v5_0/resync',
            '/nuage/api/v5_0/resync/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/resync'
        )
        self.api.add_resource(
            sim.NUSimRoutingPolicy,
            '/nuage/api/v5_0/routingpolicies',
            '/nuage/api/v5_0/routingpolicies/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/routingpolicies'
        )
        self.api.add_resource(
            sim.NUSimWANService,
            '/nuage/api/v5_0/services',
            '/nuage/api/v5_0/services/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/services'
        )
        self.api.add_resource(
            sim.NUSimSharedNetworkResource,
            '/nuage/api/v5_0/sharednetworkresources',
            '/nuage/api/v5_0/sharednetworkresources/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/sharednetworkresources'
        )
        self.api.add_resource(
            sim.NUSimSiteInfo,
            '/nuage/api/v5_0/sites',
            '/nuage/api/v5_0/sites/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/sites'
        )
        self.api.add_resource(
            sim.NUSimSPATSourcesPool,
            '/nuage/api/v5_0/spatsourcespools',
            '/nuage/api/v5_0/spatsourcespools/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/spatsourcespools'
        )
        self.api.add_resource(
            sim.NUSimSSHKey,
            '/nuage/api/v5_0/sshkeys',
            '/nuage/api/v5_0/sshkeys/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/sshkeys'
        )
        self.api.add_resource(
            sim.NUSimSSIDConnection,
            '/nuage/api/v5_0/ssidconnections',
            '/nuage/api/v5_0/ssidconnections/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/ssidconnections'
        )
        self.api.add_resource(
            sim.NUSimStaticRoute,
            '/nuage/api/v5_0/staticroutes',
            '/nuage/api/v5_0/staticroutes/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/staticroutes'
        )
        self.api.add_resource(
            sim.NUSimStatistics,
            '/nuage/api/v5_0/statistics',
            '/nuage/api/v5_0/statistics/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/statistics'
        )
        self.api.add_resource(
            sim.NUSimStatsCollectorInfo,
            '/nuage/api/v5_0/statisticscollector',
            '/nuage/api/v5_0/statisticscollector/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/statisticscollector'
        )
        self.api.add_resource(
            sim.NUSimStatisticsPolicy,
            '/nuage/api/v5_0/statisticspolicies',
            '/nuage/api/v5_0/statisticspolicies/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/statisticspolicies'
        )
        self.api.add_resource(
            sim.NUSimSubnet,
            '/nuage/api/v5_0/subnets',
            '/nuage/api/v5_0/subnets/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/subnets'
        )
        self.api.add_resource(
            sim.NUSimSubnetTemplate,
            '/nuage/api/v5_0/subnettemplates',
            '/nuage/api/v5_0/subnettemplates/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/subnettemplates'
        )
        self.api.add_resource(
            sim.NUSimSystemConfig,
            '/nuage/api/v5_0/systemconfigs',
            '/nuage/api/v5_0/systemconfigs/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/systemconfigs'
        )
        self.api.add_resource(
            sim.NUSimTCA,
            '/nuage/api/v5_0/tcas',
            '/nuage/api/v5_0/tcas/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/tcas'
        )
        self.api.add_resource(
            sim.NUSimTier,
            '/nuage/api/v5_0/tiers',
            '/nuage/api/v5_0/tiers/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/tiers'
        )
        self.api.add_resource(
            sim.NUSimTrunk,
            '/nuage/api/v5_0/trunks',
            '/nuage/api/v5_0/trunks/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/trunks'
        )
        self.api.add_resource(
            sim.NUSimUnderlay,
            '/nuage/api/v5_0/underlays',
            '/nuage/api/v5_0/underlays/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/underlays'
        )
        self.api.add_resource(
            sim.NUSimUplinkConnection,
            '/nuage/api/v5_0/uplinkconnections',
            '/nuage/api/v5_0/uplinkconnections/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/uplinkconnections'
        )
        self.api.add_resource(
            sim.NUSimUplinkRD,
            '/nuage/api/v5_0/uplinkroutedistinguishers',
            '/nuage/api/v5_0/uplinkroutedistinguishers/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/uplinkroutedistinguishers'
        )
        self.api.add_resource(
            sim.NUSimUserContext,
            '/nuage/api/v5_0/usercontexts',
            '/nuage/api/v5_0/usercontexts/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/usercontexts'
        )
        self.api.add_resource(
            sim.NUSimUser,
            '/nuage/api/v5_0/users',
            '/nuage/api/v5_0/users/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/users'
        )
        self.api.add_resource(
            sim.NUSimVCenterCluster,
            '/nuage/api/v5_0/vcenterclusters',
            '/nuage/api/v5_0/vcenterclusters/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/vcenterclusters'
        )
        self.api.add_resource(
            sim.NUSimVCenterDataCenter,
            '/nuage/api/v5_0/vcenterdatacenters',
            '/nuage/api/v5_0/vcenterdatacenters/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/vcenterdatacenters'
        )
        self.api.add_resource(
            sim.NUSimVCenterHypervisor,
            '/nuage/api/v5_0/vcenterhypervisors',
            '/nuage/api/v5_0/vcenterhypervisors/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/vcenterhypervisors'
        )
        self.api.add_resource(
            sim.NUSimVCenter,
            '/nuage/api/v5_0/vcenters',
            '/nuage/api/v5_0/vcenters/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/vcenters'
        )
        self.api.add_resource(
            sim.NUSimVirtualFirewallPolicy,
            '/nuage/api/v5_0/virtualfirewallpolicies',
            '/nuage/api/v5_0/virtualfirewallpolicies/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/virtualfirewallpolicies'
        )
        self.api.add_resource(
            sim.NUSimVirtualFirewallRule,
            '/nuage/api/v5_0/virtualfirewallrules',
            '/nuage/api/v5_0/virtualfirewallrules/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/virtualfirewallrules'
        )
        self.api.add_resource(
            sim.NUSimVirtualIP,
            '/nuage/api/v5_0/virtualips',
            '/nuage/api/v5_0/virtualips/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/virtualips'
        )
        self.api.add_resource(
            sim.NUSimVLAN,
            '/nuage/api/v5_0/vlans',
            '/nuage/api/v5_0/vlans/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/vlans'
        )
        self.api.add_resource(
            sim.NUSimVLANTemplate,
            '/nuage/api/v5_0/vlantemplates',
            '/nuage/api/v5_0/vlantemplates/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/vlantemplates'
        )
        self.api.add_resource(
            sim.NUSimVMInterface,
            '/nuage/api/v5_0/vminterfaces',
            '/nuage/api/v5_0/vminterfaces/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/vminterfaces'
        )
        self.api.add_resource(
            sim.NUSimVM,
            '/nuage/api/v5_0/vms',
            '/nuage/api/v5_0/vms/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/vms'
        )
        self.api.add_resource(
            sim.NUSimVNFCatalog,
            '/nuage/api/v5_0/vnfcatalogs',
            '/nuage/api/v5_0/vnfcatalogs/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/vnfcatalogs'
        )
        self.api.add_resource(
            sim.NUSimVNFDescriptor,
            '/nuage/api/v5_0/vnfdescriptors',
            '/nuage/api/v5_0/vnfdescriptors/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/vnfdescriptors'
        )
        self.api.add_resource(
            sim.NUSimVNFDomainMapping,
            '/nuage/api/v5_0/vnfdomainmappings',
            '/nuage/api/v5_0/vnfdomainmappings/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/vnfdomainmappings'
        )
        self.api.add_resource(
            sim.NUSimVNFInterfaceDescriptor,
            '/nuage/api/v5_0/vnfinterfacedescriptors',
            '/nuage/api/v5_0/vnfinterfacedescriptors/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/vnfinterfacedescriptors'
        )
        self.api.add_resource(
            sim.NUSimVNFInterface,
            '/nuage/api/v5_0/vnfinterfaces',
            '/nuage/api/v5_0/vnfinterfaces/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/vnfinterfaces'
        )
        self.api.add_resource(
            sim.NUSimVNFMetadata,
            '/nuage/api/v5_0/vnfmetadatas',
            '/nuage/api/v5_0/vnfmetadatas/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/vnfmetadatas'
        )
        self.api.add_resource(
            sim.NUSimVNF,
            '/nuage/api/v5_0/vnfs',
            '/nuage/api/v5_0/vnfs/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/vnfs'
        )
        self.api.add_resource(
            sim.NUSimVNFThresholdPolicy,
            '/nuage/api/v5_0/vnfthresholdpolicies',
            '/nuage/api/v5_0/vnfthresholdpolicies/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/vnfthresholdpolicies'
        )
        self.api.add_resource(
            sim.NUSimVPNConnection,
            '/nuage/api/v5_0/vpnconnections',
            '/nuage/api/v5_0/vpnconnections/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/vpnconnections'
        )
        self.api.add_resource(
            sim.NUSimVPortMirror,
            '/nuage/api/v5_0/vportmirrors',
            '/nuage/api/v5_0/vportmirrors/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/vportmirrors'
        )
        self.api.add_resource(
            sim.NUSimVPort,
            '/nuage/api/v5_0/vports',
            '/nuage/api/v5_0/vports/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/vports'
        )
        self.api.add_resource(
            sim.NUSimVRSAddressRange,
            '/nuage/api/v5_0/vrsaddressranges',
            '/nuage/api/v5_0/vrsaddressranges/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/vrsaddressranges'
        )
        self.api.add_resource(
            sim.NUSimVCenterVRSConfig,
            '/nuage/api/v5_0/vrsconfigs',
            '/nuage/api/v5_0/vrsconfigs/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/vrsconfigs'
        )
        self.api.add_resource(
            sim.NUSimVRSMetrics,
            '/nuage/api/v5_0/vrsmetrics',
            '/nuage/api/v5_0/vrsmetrics/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/vrsmetrics'
        )
        self.api.add_resource(
            sim.NUSimVRSRedeploymentpolicy,
            '/nuage/api/v5_0/vrsredeploymentpolicies',
            '/nuage/api/v5_0/vrsredeploymentpolicies/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/vrsredeploymentpolicies'
        )
        self.api.add_resource(
            sim.NUSimVRS,
            '/nuage/api/v5_0/vrss',
            '/nuage/api/v5_0/vrss/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/vrss'
        )
        self.api.add_resource(
            sim.NUSimVSC,
            '/nuage/api/v5_0/vscs',
            '/nuage/api/v5_0/vscs/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/vscs'
        )
        self.api.add_resource(
            sim.NUSimVSD,
            '/nuage/api/v5_0/vsds',
            '/nuage/api/v5_0/vsds/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/vsds'
        )
        self.api.add_resource(
            sim.NUSimVsgRedundantPort,
            '/nuage/api/v5_0/vsgredundantports',
            '/nuage/api/v5_0/vsgredundantports/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/vsgredundantports'
        )
        self.api.add_resource(
            sim.NUSimVSP,
            '/nuage/api/v5_0/vsps',
            '/nuage/api/v5_0/vsps/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/vsps'
        )
        self.api.add_resource(
            sim.NUSimWirelessPort,
            '/nuage/api/v5_0/wirelessports',
            '/nuage/api/v5_0/wirelessports/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/wirelessports'
        )
        self.api.add_resource(
            sim.NUSimZFBAutoAssignment,
            '/nuage/api/v5_0/zfbautoassignments',
            '/nuage/api/v5_0/zfbautoassignments/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/zfbautoassignments'
        )
        self.api.add_resource(
            sim.NUSimZFBRequest,
            '/nuage/api/v5_0/zfbrequests',
            '/nuage/api/v5_0/zfbrequests/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/zfbrequests'
        )
        self.api.add_resource(
            sim.NUSimZone,
            '/nuage/api/v5_0/zones',
            '/nuage/api/v5_0/zones/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/zones'
        )
        self.api.add_resource(
            sim.NUSimZoneTemplate,
            '/nuage/api/v5_0/zonetemplates',
            '/nuage/api/v5_0/zonetemplates/<entity_id>',
            '/nuage/api/v5_0/<parent_type>/<parent_id>/zonetemplates'
        )
        

        self.app.run(host='0.0.0.0', port=args.port, debug=(log_level == 'DEBUG'))


def main():
    NuageVSDSim()